@@ -100,6 +100,9 @@
 /* Sent to users when they encounter mode +R */
 #define DEFAULT_NS_REGISTER_URL "http://docs.dal.net/docs/nsemail.html"
 
+/* SpamFilter KB/Help URL (sent to users when they're getting warned/blocked by spamfilter) */
+#define DEFAULT_SPAMFILTER_URL "http://www.dal.net/kb/view.php?kb=411"
+
 /* self explanitory */
 #define DEFAULT_MAXCHANNELSPERUSER 10
 
@@ -382,6 +385,18 @@
 # define MAX_AWAY_COUNT 5
 #endif
 
+/* NO_UMODE_H_FLOOD
+ * Prevent users from setting and unsetting user mode H for hostmasking
+ * too often, as this could result in a WATCH flood. Default setting
+ * allows 2 mode changes in 5 minutes.
+ */
+#define NO_UMODE_H_FLOOD
+
+#ifdef NO_UMODE_H_FLOOD
+#define MAX_UMODE_H_TIME 300  /* time in seconds */
+#define MAX_UMODE_H_COUNT 2
+#endif
+
 /*
  * WARN_NO_NLINE Define this if you want ops to get noticed about
  * "things" trying to connect as servers that don't have N: lines.
@@ -852,6 +867,38 @@
  * -wd */
 #define DONT_CHECK_QLINE_REMOTE
 
+/* USER_HOSTMASKING is used to enable host-masking for users.
+ * If you are connected to the main DALnet network, you should not
+ * have this enabled yet.
+ */
+#define USER_HOSTMASKING
+
+/* SPAMFILTER is used to enable spam filtering on the network.
+ * If you are connected to the main DALnet network, you should
+ * have this enabled.
+ */
+#define SPAMFILTER
+
+/*
+ * DEFAULT_OPER_SPAMFILTER_DISABLED - default your opers to +P, which disables spamfilter checking.  This
+ * is strongly recommended so that users can report potential spam.
+ */
+#ifdef SPAMFILTER
+#define DEFAULT_OPER_SPAMFILTER_DISABLED
+#endif
+
+/*
+ * USE_HALFOPS is used to enable half-ops support
+ * If you are connected to the main DALnet network, you should
+ * have this enabled.
+ */
+#define USE_HALFOPS
+
+/*
+ * Don't allow local clients to use +h/-h until all servers and services are upgraded.
+ */
+#undef NO_LOCAL_CMODE_h
+
 /* ------------------------- END CONFIGURATION SECTION -------------------- */
 #ifdef APOLLO
 #define RESTARTING_SYSTEMCALLS
