@@ -99,6 +99,7 @@
 #define MSG_SVSHOLD  "SVSHOLD"		/* SVSHOLD */
 #define MSG_AKILL    "AKILL"     	/* AKILL */
 #define MSG_RAKILL   "RAKILL"    	/* RAKILL */
+#define MSG_NBANRESET "NBANRESET" 	/* NBANRESET */
 #define MSG_SILENCE  "SILENCE"   	/* SILENCE */
 #define MSG_WATCH    "WATCH"     	/* WATCH */
 #define MSG_SQLINE   "SQLINE" 		/* SQLINE */
@@ -125,6 +126,7 @@
 #define MSG_CHANKILL "CHANKILL"     /* CHANKILL */
 #define MSG_SVSHOST  "SVSHOST"      /* SVSHOST */
 #define MSG_SVSTAG   "SVSTAG"       /* SVSTAG */
+#define MSG_SVSUHM   "SVSUHM"       /* SVSUHM */
 #define MSG_PUT      "PUT"          /* PUT */
 #define MSG_POST     "POST"         /* POST */
 #define MSG_CHECK    "CHECK"        /* CHECK */
@@ -137,6 +139,7 @@ extern int  m_kline(aClient *, aClient *, int, char **);
 extern int  m_unkline(aClient *, aClient *, int, char **);
 extern int  m_akill(aClient *, aClient *, int, char **);
 extern int  m_rakill(aClient *, aClient *, int, char **);
+extern int  m_nbanreset(aClient *, aClient *, int, char **);
 extern int  m_locops(aClient *, aClient *, int, char **);
 extern int  m_private(aClient *, aClient *, int, char **);
 extern int  m_topic(aClient *, aClient *, int, char **);
@@ -225,10 +228,17 @@ extern int  m_chankill(aClient *, aClient *, int, char **);
 extern int  m_svshost(aClient *, aClient *, int, char **);
 extern int  m_svsnoop(aClient *, aClient *, int, char **);
 extern int  m_svstag(aClient *, aClient *, int, char **);
+extern int  m_svsuhm(aClient *, aClient *, int, char **);
 extern int  m_put(aClient *, aClient *, int, char **);
 extern int  m_post(aClient *, aClient *, int, char **);
 extern int  m_check(aClient *, aClient *, int, char **);
 extern int  m_webirc(aClient *, aClient *, int, char **);
+extern int m_spamops(aClient *, aClient *, int, char **);
+extern int m_sf(aClient *, aClient *, int, char **);
+extern int m_aj(aClient *, aClient *, int, char **);
+extern int m_sjr(aClient *, aClient *, int, char **, AliasInfo *);
+extern int m_svsxcf(aClient *, aClient *, int, char **);
+extern int m_svsctrl(aClient *, aClient *, int, char **);
 
 /* aliastab indexes */
 #define AII_NS  0
@@ -329,6 +339,7 @@ struct Message msgtab[] =
     {MSG_SVSHOLD,  m_svshold,  MAXPARA, 0,        0},
     {MSG_AKILL,    m_akill,    MAXPARA, 0,        0},
     {MSG_RAKILL,   m_rakill,   MAXPARA, 0,        0},
+    {MSG_NBANRESET,m_nbanreset,      1, 0,        0},
     {MSG_SILENCE,  m_silence,  MAXPARA, 0,        0},
     {MSG_WATCH,    m_watch,          1, 0,        0},
     {MSG_DCCALLOW, m_dccallow,       1, 0,        0},
@@ -355,12 +366,18 @@ struct Message msgtab[] =
     {MSG_SVSHOST,  m_svshost,  MAXPARA, 0,        0},
     {MSG_SVSNOOP,  m_svsnoop,  MAXPARA, 0,        0},
     {MSG_SVSTAG,   m_svstag,   MAXPARA, 0,        0},
+    {MSG_SVSUHM,   m_svsuhm,   MAXPARA, 0,        0},
     {MSG_PUT,      m_put,      2,       MF_UNREG, 0},
     {MSG_POST,     m_post,     2,       MF_UNREG, 0},
     {MSG_CHECK,    m_check,    MAXPARA, 0,        0},
     {MSG_LUSERSLOCK, m_luserslock, MAXPARA, 0,       0},
     {MSG_LINKSCONTROL, m_linkscontrol, MAXPARA, 0,      0},
-
+    {"SPAMOPS",    m_spamops,  MAXPARA, 0,        0},
+    {"SF",         m_sf,       MAXPARA, 0,        0},
+    {"SVSXCF",     m_svsxcf,   MAXPARA, 0,        0},
+    {"SVSCTRL",    m_svsctrl,  MAXPARA, 0,        0},
+    {"AJ",         m_aj,       MAXPARA, 0,        0},
+    {"SJR",        m_sjr,      MAXPARA, MF_ALIAS, AII_NS},
     {MSG_WEBIRC,   m_webirc,   MAXPARA, MF_UNREG, 0},
     { 0 }
 };
