@@ -173,7 +173,6 @@ typedef struct SServicesTag ServicesTag;
 #define	BOOT_OPER	 16
 #define BOOT_STDERR	 64
 #define	STAT_LOG	 -6	/* logfile for -x */
-#define	STAT_MASTER	 -5	/* Local ircd master before identification */
 #define	STAT_CONNECTING	 -4
 #define	STAT_HANDSHAKE	 -3
 #define	STAT_ME		 -2
@@ -189,13 +188,11 @@ typedef struct SServicesTag ServicesTag;
 #define	IsConnecting(x)		((x)->status == STAT_CONNECTING)
 #define	IsHandshake(x)		((x)->status == STAT_HANDSHAKE)
 #define	IsMe(x)			((x)->status == STAT_ME)
-#define	IsUnknown(x)		((x)->status == STAT_UNKNOWN || \
-				 (x)->status == STAT_MASTER)
+#define	IsUnknown(x)		((x)->status == STAT_UNKNOWN)
 #define	IsServer(x)		((x)->status == STAT_SERVER)
 #define	IsClient(x)		((x)->status == STAT_CLIENT)
 #define	IsLog(x)		((x)->status == STAT_LOG)
 
-#define	SetMaster(x)		((x)->status = STAT_MASTER)
 #define	SetConnecting(x)	((x)->status = STAT_CONNECTING)
 #define	SetHandshake(x)		((x)->status = STAT_HANDSHAKE)
 #define	SetMe(x)		((x)->status = STAT_ME)
@@ -241,6 +238,7 @@ typedef struct SServicesTag ServicesTag;
 #define FLAGS_ZIPPED_IN	   0x4000000  /* This link is gzipped. */
 #define FLAGS_ZIPPED_OUT   0x8000000 /* This link is gzipped. */
 #define FLAGS_SSL          0x10000000 /* client is using SSL */
+#define FLAGS_SPOOFED      0x20000000 /* User's host was spoofed (via SVSHOST) */
 
 /* Capabilities of the ircd or clients */
 
@@ -297,6 +295,7 @@ typedef struct SServicesTag ServicesTag;
 #define UMODE_f     0x00100	/* umode +f - Server flood messages */
 #define UMODE_y     0x00200	/* umode +y - Stats/links */
 #define UMODE_d     0x00400	/* umode +d - Debug info */
+#define UMODE_P     0x00800     /* umode +P - User wants extra privacy (no spamfilter) */
 #define UMODE_g     0x01000	/* umode +g - Globops */
 #define UMODE_b     0x02000	/* umode +b - Chatops */
 #define UMODE_a     0x04000	/* umode +a - Services Admin */
@@ -315,6 +314,10 @@ typedef struct SServicesTag ServicesTag;
 #define UMODE_I     0x8000000   /* umode +I - invisible oper (masked) */
 #define UMODE_S     0x10000000  /* umode +S - User is using SSL */
 #define UMODE_C     0x20000000  /* umode +C - User is only accepting private messages from users who share a common channel with them */
+#define UMODE_H     0x40000000  /* umode +H - User is host-masked */
+
+/* WARNING: Do not add any values greater than 0x40000000 unless you change Client->umode to unsigned long
+            (and change everything else to support it) -Kobi & xPsycho. */
 
 /* for sendto_ops_lev */
 
@@ -337,13 +340,13 @@ typedef struct SServicesTag ServicesTag;
  *  that mode will be 'silent.'
  */
 
-#define SEND_UMODES (UMODE_a|UMODE_i|UMODE_o|UMODE_r|UMODE_A|UMODE_I|UMODE_R|UMODE_S|UMODE_C)
+#define SEND_UMODES (UMODE_a|UMODE_i|UMODE_o|UMODE_r|UMODE_A|UMODE_I|UMODE_R|UMODE_S|UMODE_C|UMODE_H|UMODE_P)
 #define ALL_UMODES (SEND_UMODES|UMODE_b|UMODE_c|UMODE_d|UMODE_e|UMODE_f|\
                     UMODE_g|UMODE_h|UMODE_j|UMODE_k|UMODE_m|UMODE_n|UMODE_s|\
                     UMODE_w|UMODE_y|UMODE_F|UMODE_K|UMODE_O)
 
 /* modes users can set themselves */
-#define USER_UMODES (UMODE_i|UMODE_k|UMODE_w|UMODE_s|UMODE_R|UMODE_C)
+#define USER_UMODES (UMODE_i|UMODE_k|UMODE_w|UMODE_s|UMODE_R|UMODE_C|UMODE_H|UMODE_P)
 
 /* modes only opers can have */
 #define OPER_UMODES (UMODE_a|UMODE_b|UMODE_c|UMODE_d|UMODE_e|UMODE_f|UMODE_g|\
@@ -377,6 +380,8 @@ typedef struct SServicesTag ServicesTag;
 #define IsUmodeS(x)		((x)->umode & UMODE_S)
 #define	IsUmodes(x)		((x)->umode & UMODE_s)
 #define	IsUmodeI(x)		((x)->umode & UMODE_I)
+#define	IsUmodeH(x)		((x)->umode & UMODE_H)
+#define	IsUmodeP(x)		((x)->umode & UMODE_P)
 #define IsNoNonReg(x)           ((x)->umode & UMODE_R)
 #define IsWSquelch(x)           ((x)->umode & UMODE_x)
 #define IsSSquelch(x)           ((x)->umode & UMODE_X)
@@ -495,7 +500,7 @@ typedef struct SServicesTag ServicesTag;
                          OFLAG_UNKLINE|OFLAG_LNOTICE|OFLAG_UMODEc|OFLAG_UMODEf|OFLAG_UMODEd|\
                          OFLAG_UMODEb|OFLAG_UMODEy)
 #define OFLAG_GLOBAL	(OFLAG_LOCAL|OFLAG_GROUTE|OFLAG_GKILL|OFLAG_GNOTICE)
-#define OFLAG_ISGLOBAL	(OFLAG_GROUTE|OFLAG_GKILL|OFLAG_GNOTICE|OFLAG_SADMIN|OFLAG_UMODEF)
+#define OFLAG_ISGLOBAL	(OFLAG_GROUTE|OFLAG_GKILL|OFLAG_GNOTICE|OFLAG_SADMIN)
 #define OPCanZline(x)	        ((x)->oflag & OFLAG_SADMIN)
 #define OPCanRehash(x)	        ((x)->oflag & OFLAG_REHASH)
 #define OPCanDie(x)	        ((x)->oflag & OFLAG_DIE)
@@ -598,10 +603,14 @@ typedef struct Whowas
     char        name[NICKLEN + 1];
     char        username[USERLEN + 1];
     char        hostname[HOSTLEN + 1];
+#ifdef USER_HOSTMASKING
+    char        mhostname[HOSTLEN + 1];
+    char        hostip[HOSTIPLEN + 1];
+#endif
     char       *servername;
     char        realname[REALLEN + 1];
     time_t      logoff;
-    unsigned int umode;
+    long umode;
     struct Client *online;  /* Pointer to new nickname for chasing or NULL */
     
     struct Whowas *next;    /* for hash table... */
@@ -675,6 +684,8 @@ typedef struct Whowas
 #define FLAGS_WGMON     (FLAGS_WGMONURL|FLAGS_WGMONHOST)
 #define FLAGS_SHOWLINKS 0x0040
 #define FLAGS_SPLITOPOK 0x0080
+#define FLAGS_REMREHOK  0x0100
+
 
 /* flags for connects */
 
@@ -837,6 +848,9 @@ struct User
     int         joined;        /* number of channels joined */
     char        username[USERLEN + 1];
     char        host[HOSTLEN + 1];
+#ifdef USER_HOSTMASKING
+    char        mhost[HOSTLEN + 1];
+#endif
     char       *server;        /* pointer to scached server name */
     unsigned int servicetype;  /* set by SVSMODE +T */
     unsigned long servicestamp; /* set by SVSMODE +d */
@@ -890,7 +904,11 @@ struct Client
     time_t      since;      /* last time we parsed something */
     ts_val      tsinfo;     /* TS on the nick, SVINFO on servers */
     long        flags;      /* client flags */
-    long        umode;      /* We can illeviate overflow this way */
+    long        umode;      /* We can illeviate overflow this way
+                               Note: if you change this, you need to also
+                               change struct SearchOptions,
+                               struct Whowas and struct SServicesTag -Kobi.
+                             */
     aClient    *from;       /* == self, if Local Client, *NEVER* NULL! */
     aClient    *uplink;     /* this client's uplink to the network */
     int         fd;         /* >= 0, for local clients */
@@ -1012,7 +1030,11 @@ struct Client
     time_t	    alas;	/* last time of away set */
     int	    acount;	        /* count of away settings */
 #endif
-    
+#ifdef NO_UMODE_H_FLOOD
+    time_t  last_umodeh_change;         /* last time of umode H change */
+    int	    number_of_umodeh_changes;	/* count of umode H changes */
+#endif
+
     char        sockhost[HOSTLEN + 1];	/* This is the host name from
 					 * the socket and after which the 
 					 * connection was accepted. */
@@ -1179,6 +1201,9 @@ struct ChanLink
     struct ChanLink *next;
     aClient *cptr;
     int flags;
+    time_t when;
+    int last_message_number;    /* Number of messages sent to channel within max_messages_time */
+    time_t last_message_time;   /* When last message was sent to channel. -Holbrook */
     unsigned int banserial;     /* used for bquiet cache */
 };
 
@@ -1247,6 +1272,15 @@ struct Channel
     int         jrw_debt_ctr;   /* join rate warning: in-debt counter */
     int         jrw_debt_ts;    /* join rate warning: debt begin timestamp */
     unsigned int banserial;     /* used for bquiet cache */
+    int join_connect_time;      /* Number of seconds the user must be online to be able to join */
+    int talk_connect_time;      /* Number of seconds the user must be online to be able to talk on the channel */
+    int talk_join_time;         /* Number of seconds the user must be on the channel to be able to tlak on the channel */
+    int max_bans;               /* Maximum number of bans ops can add (default: MAXBANS) */
+    int max_invites;            /* Maximum number of invites ops can add (default: MAXINVITELIST) */
+    int max_messages;           /* Maximum number of messages can be sent within max_messages_time */
+    int max_messages_time;      /* Number of seconds for how many messages can be sent, e.g., 5:10, 5 messages in 10 seconds */
+    char *greetmsg;             /* Special greeting message */
+    int xflags;                 /* The eXtended channel flags */
 };
 
 #define	TS_CURRENT	5	/* current TS protocol version */
@@ -1261,6 +1295,7 @@ struct Channel
 #define	CHFL_VOICE      0x0002	/* the power to speak */
 #define	CHFL_DEOPPED 	0x0004	/* deopped by us, modes need to be bounced */
 #define	CHFL_BANNED     0x0008  /* is banned */
+#define	CHFL_HALFOP     0x0010  /* channel half-op */
 
 /* ban mask types */
 
@@ -1291,6 +1326,7 @@ struct Channel
 #define MODE_JOINRATE	0x40000
 #define MODE_SSLONLY	0x80000
 #define MODE_AUDITORIUM 0x100000
+#define MODE_PRIVACY    0x200000 /* cmode +P - Channel wants extra privacy (no spamfilter) */
 
 /* mode flags which take another parameter (With PARAmeterS) */
 
@@ -1314,7 +1350,7 @@ struct Channel
 
 /* channel visible */
 
-#define	ShowChannel(v,c)	(PubChannel(c) || IsMember((v),(c)))
+#define	ShowChannel(v,c)	(!SecretChannel(c) || IsMember((v),(c)))
 #define	PubChannel(x)		((!x) || ((x)->mode.mode &\
                                  (MODE_PRIVATE | MODE_SECRET)) == 0)
 
@@ -1323,6 +1359,31 @@ struct Channel
 
 #define	IsChannelName(name) ((name) && (*(name) == '#'))
 
+/* Extended Channel Flags */
+#define XFLAG_SET               0x0001 /* This is to indicate the channel has enabled xflag(s) */
+#define XFLAG_NO_NOTICE         0x0002
+#define XFLAG_NO_CTCP           0x0004
+#define XFLAG_NO_PART_MSG       0x0008
+#define XFLAG_NO_QUIT_MSG       0x0010
+#define XFLAG_EXEMPT_IDENTD     0x0020
+#define XFLAG_EXEMPT_REGISTERED 0x0040
+#define XFLAG_EXEMPT_INVITES    0x0080
+#define XFLAG_EXEMPT_OPPED      0x0100
+#define XFLAG_EXEMPT_VOICED     0x0200
+#define XFLAG_HIDE_MODE_LISTS   0x0400
+#define XFLAG_USER_VERBOSE      0x0800
+#define XFLAG_OPER_VERBOSE      0x1000
+#define XFLAG_SJR               0x2000 /* Services join request */
+#define XFLAG_NO_NICK_CHANGE    0x4000
+#define XFLAG_NO_UTF8           0x8000
+#define XFLAG_EXEMPT_WEBIRC     0x10000
+
+struct FlagList
+{
+    char *option;
+    int flag;
+};
+
 /* Misc macros */
 
 #define	BadPtr(x) (!(x) || (*(x) == '\0'))
@@ -1442,7 +1503,7 @@ struct ListOptions
 
 typedef struct SearchOptions 
 {
-    int umodes;
+    long umodes;
     char *nick;
     char *user;
     char *host;
@@ -1477,6 +1538,8 @@ typedef struct SearchOptions
     unsigned show_chan:1;
     unsigned search_chan:1;
     unsigned ip_show:1;
+    unsigned realhost_show:1;
+    unsigned maskhost_show:1;
     unsigned client_type_plus:1;
 } SOpts;
 
