@@ -33,19 +33,22 @@ extern void fakelinkserver_update(char *, char *);
 extern void fakeserver_sendserver(aClient *);
 extern void fakelusers_sendlock(aClient *);
 extern void reset_sock_opts(int, int);
+extern void spamfilter_sendserver(aClient *acptr);
 extern int user_modes[];
+extern int uhm_type;
+extern int uhm_umodeh;
 
 /* internal functions */
 
 static void sendnick_TS(aClient *cptr, aClient *acptr)
 {
     ServicesTag *servicestag;
-    static char ubuf[30];
+    static char ubuf[54];
     int *s, flag, i;
 
     if (IsPerson(acptr))
     {
-        send_umode(NULL, acptr, 0, SEND_UMODES, ubuf);
+        send_umode(NULL, acptr, 0, SEND_UMODES, ubuf, sizeof(ubuf));
         if (!*ubuf)     /* trivial optimization - Dianora */
         {
             ubuf[0] = '+';
@@ -81,6 +84,10 @@ static void sendnick_TS(aClient *cptr, aClient *acptr)
             sendto_one(cptr, "SVSTAG %s %ld %d %s :%s", acptr->name, acptr->tsinfo, servicestag->raw,
                        ubuf, servicestag->tag);
         }
+#ifdef USER_HOSTMASKING
+        if(acptr->flags & FLAGS_SPOOFED)
+            sendto_one(cptr, "SVSHOST %s %s", acptr->name, acptr->user->mhost);
+#endif
     }
 }
 
@@ -94,9 +101,14 @@ do_server_estab(aClient *cptr)
     /* "refresh" inpath with host  */
     char *inpath = get_client_name(cptr, HIDEME);
 
+    /*
+     * Incoming connections have unknown status,
+     * outbound connections have status of connecting
+     */
+    if (IsUnknown(cptr)) Count.unknown--;
+
     SetServer(cptr);
 
-    Count.unknown--;
     Count.server++;
     Count.myserver++;
 
@@ -253,6 +265,15 @@ do_server_estab(aClient *cptr)
     /* Send out fake server list and other 'fake' stuff */
     fakeserver_sendserver(cptr);
 
+#ifdef SPAMFILTER
+    /* Send the spamfilters... */
+    spamfilter_sendserver(cptr);
+#endif
+
+    /* Send UHM (user host-masking) type */
+    if(confopts & FLAGS_HUB)
+        sendto_one(cptr, "SVSUHM %d", uhm_type);
+
     /* send clone list */
     clones_send(cptr);
 
@@ -358,7 +379,10 @@ m_server_estab(aClient *cptr)
     {
         ircstp->is_ref++;
         sendto_one(cptr, "ERROR :Wrong link password");
-        sendto_ops("Link %s dropped, wrong password", inpath);
+        sendto_gnotice("from %s: Link %s dropped, wrong password",
+                       me.name, inpath);
+        sendto_serv_butone(cptr, ":%s GNOTICE :Link %s dropped, wrong"
+                           " password", me.name, inpath);
         return exit_client(cptr, cptr, cptr, "Bad Password");
     }
     memset(cptr->passwd, '\0', sizeof(cptr->passwd));
@@ -416,7 +440,7 @@ m_server_estab(aClient *cptr)
                    my_name_for_link(me.name, aconn),
                    (me.info[0]) ? (me.info) : "IRCers United");
     }
-    else 
+    else
     {
         s = (char *) strchr(aconn->host, '@');
         *s = '\0';      /* should never be NULL -- wanna bet? -Dianora */
@@ -629,7 +653,7 @@ int m_server(aClient *cptr, aClient *sptr, int parc, char *parv[])
                 return exit_client(cptr, cptr, cptr, "No Connect block");
         }
     }
-    
+
     /* already linked server */
     if (!IsServer(cptr))
         return 0;
@@ -789,7 +813,7 @@ int m_server(aClient *cptr, aClient *sptr, int parc, char *parv[])
 
 /* m_dkey
  * lucas's code, i assume.
- * moved here from s_serv.c due to its integration in the encrypted 
+ * moved here from s_serv.c due to its integration in the encrypted
  * server negotiation stuffs. -epi
  */
 
@@ -909,4 +933,3 @@ int m_dkey(aClient *cptr, aClient *sptr, int parc, char *parv[])
 #endif
     return 0;
 }
-
