@@ -34,7 +34,7 @@
 /* Internally defined stuffs */
 SOpts wsopts;
 int build_searchopts(aClient *, int, char **);
-int chk_who(aClient *, int);
+int chk_who(aClient *, aClient *, int);
 
 /* Externally defined stuffs */
 extern int user_modes[];
@@ -45,7 +45,11 @@ int build_searchopts(aClient *sptr, int parc, char *parv[])
 {
   static char *who_oper_help[] =
   {
-      "/WHO [+|-][acghilmnstuCM] [args]",
+#ifdef USER_HOSTMASKING
+      "/WHO [+|-][acghilmnstuCHIMR] [args]",
+#else
+      "/WHO [+|-][acghilmnstuCIM] [args]",
+#endif
       "Flags are specified like channel modes,",
       "The flags cghimnsu all have arguments",
       "Flags are set to a positive check by +, a negative check by -",
@@ -73,6 +77,10 @@ int build_searchopts(aClient *sptr, int parc, char *parv[])
       "Flag C: show first visible channel user is in",
       "Flag M: check for user in channels I am a member of",
       "Flag I: always show IPs instead of hosts",
+#ifdef USER_HOSTMASKING
+      "Flag H: show the masked host even if the user's host is not masked (umode -H)",
+      "Flag R: show the real host even if the user's host is masked (umode +H)",
+#endif
       NULL
   };
 
@@ -202,7 +210,7 @@ int build_searchopts(aClient *sptr, int parc, char *parv[])
       else
       {
 	  /* The argument could be either a hostname or a nickname. */
-	  if (strchr(parv[0], '.'))
+	  if (strchr(parv[0], '.') || strchr(parv[0], ':'))
 	  {
 	      wsopts.host_plus = 1;
 	      wsopts.host = parv[0];
@@ -346,6 +354,24 @@ int build_searchopts(aClient *sptr, int parc, char *parv[])
           }
           wsopts.ip_show = change;
           break; 
+#ifdef USER_HOSTMASKING
+      case 'H':
+          if(!IsAnOper(sptr))
+          {
+              sendto_one(sptr, getreply(ERR_NOPRIVILEGES), me.name, parv[0]);
+              return 0;
+          }
+          wsopts.maskhost_show = change;
+          break;
+      case 'R':
+          if(!IsAnOper(sptr))
+          {
+              sendto_one(sptr, getreply(ERR_NOPRIVILEGES), me.name, parv[0]);
+              return 0;
+          }
+          wsopts.realhost_show = change;
+          break; 
+#endif
       case 'i':
           if(parv[args]==NULL)
           {
@@ -559,7 +585,7 @@ int (*uchkfn)(char *, char *);
 int (*hchkfn)(char *, char *);
 int (*ichkfn)(char *, char *);
 
-int chk_who(aClient *ac, int showall)
+int chk_who(aClient *ac, aClient *sptr, int showall)
 {
     if(!IsClient(ac))
 	return 0;
@@ -603,15 +629,32 @@ int chk_who(aClient *ac, int showall)
 	    return 0;
 
     if(wsopts.nick!=NULL)
-	if((wsopts.nick_plus && nchkfn(wsopts.nick, ac->name)) ||
-	   (!wsopts.nick_plus && !nchkfn(wsopts.nick, ac->name)))
-	    return 0;
-    
-    if(wsopts.host!=NULL)
-	if((wsopts.host_plus && hchkfn(wsopts.host, ac->user->host)) ||
-	   (!wsopts.host_plus && !hchkfn(wsopts.host, ac->user->host)))
-	    return 0;
+        if((wsopts.nick_plus && nchkfn(wsopts.nick, ac->name)) ||
+           (!wsopts.nick_plus && !nchkfn(wsopts.nick, ac->name)))
+            return 0;
 
+    if(wsopts.host!=NULL)
+    {
+#ifdef USER_HOSTMASKING
+        if(IsAnOper(sptr))
+        {
+	    if((wsopts.host_plus && hchkfn(wsopts.host, ac->user->host) && hchkfn(wsopts.host, ac->user->mhost)) ||
+	       (!wsopts.host_plus && (!hchkfn(wsopts.host, ac->user->host) || !hchkfn(wsopts.host, ac->user->mhost))))
+	        return 0;
+        }
+        else
+        {
+	    if((wsopts.host_plus && hchkfn(wsopts.host, IsUmodeH(ac)?ac->user->mhost:ac->user->host)) ||
+	       (!wsopts.host_plus && !hchkfn(wsopts.host, IsUmodeH(ac)?ac->user->mhost:ac->user->host)))
+	        return 0;
+        }
+#else
+        if((wsopts.host_plus && hchkfn(wsopts.host, ac->user->host)) ||
+           (!wsopts.host_plus && !hchkfn(wsopts.host, ac->user->host)))
+            return 0;
+#endif
+    }
+    
     if(wsopts.cidr_plus)
 	if(ac->ip_family != wsopts.cidr_family ||
 	   bitncmp(&ac->ip, &wsopts.cidr_ip, wsopts.cidr_bits) != 0)
@@ -629,7 +672,7 @@ int chk_who(aClient *ac, int showall)
     /*
      * For the below options, a value of two means '+', 
      * a value of 1 means '-', and a value of 0 means
-     * not speficied. 
+     * not specified.
      */
 
     if(wsopts.ts_value == 2 && /* +t */
@@ -683,7 +726,11 @@ inline char *first_visible_channel(aClient *cptr, aClient *sptr)
 #define MAXWHOREPLIES 200
 #define WHO_HOPCOUNT(s, a) ( ( (IsULine((a)) || IsUmodeI((a))) && !IsAnOper((s)) ) ? 0 : a->hopcount)
 #define WHO_SERVER(s ,a) ((IsUmodeI((a)) && !IsAnOper((s))) ? HIDDEN_SERVER_NAME : a->user->server)
-#define WHO_HOST(a) ((wsopts.ip_show) ? (a)->hostip : (a)->user->host)
+#ifdef USER_HOSTMASKING
+#define WHO_HOST(s, a) ((wsopts.ip_show) ? (a)->hostip : ((IsUmodeH((a)) && !wsopts.realhost_show) || wsopts.maskhost_show) ? (a)->user->mhost : (a)->user->host)
+#else
+#define WHO_HOST(s, a) ((wsopts.ip_show) ? (a)->hostip : (a)->user->host)
+#endif
 int m_who(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
     aClient *ac;
@@ -729,7 +776,7 @@ int m_who(aClient *cptr, aClient *sptr, int parc, char *parv[])
     if(wsopts.channel!=NULL)
     {
 	if(IsMember(sptr,wsopts.channel) && (!(wsopts.channel->mode.mode & MODE_AUDITORIUM) ||
-           is_chan_opvoice(sptr, wsopts.channel)))
+           is_chan_opvoice(sptr, wsopts.channel) || IsAnOper(sptr)))
 	    showall=1;
 	else if(SecretChannel(wsopts.channel) && IsAdmin(sptr))
 	    showall=1;
@@ -743,7 +790,7 @@ int m_who(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	    {
 		ac=cm->cptr;
 		i=0;
-		if(!chk_who(ac,showall))
+		if(!chk_who(ac,sptr,showall))
 		    continue;
 		/* If we have channel flags set, verify they match */
 		if(wsopts.channelflags && ((cm->flags & wsopts.channelflags) == 0))
@@ -760,7 +807,7 @@ int m_who(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		status[++i]=0;
 		sendto_one(sptr, getreply(RPL_WHOREPLY), me.name, sptr->name,
 			   wsopts.channel->chname, ac->user->username,
-			   WHO_HOST(ac), WHO_SERVER(sptr, ac), ac->name, status,
+			   WHO_HOST(sptr,ac), WHO_SERVER(sptr, ac), ac->name, status,
 			   WHO_HOPCOUNT(sptr, ac),
 			   ac->info);
 	    }
@@ -776,7 +823,7 @@ int m_who(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	ac=find_person(wsopts.nick,NULL);
 	if(ac!=NULL)
 	{
-	    if(!chk_who(ac,1))
+	    if(!chk_who(ac,sptr,1))
 	    {
 		sendto_one(sptr, getreply(RPL_ENDOFWHO), me.name, sptr->name,
 			   wsopts.host!=NULL ? wsopts.host : wsopts.nick, "WHO");
@@ -790,7 +837,7 @@ int m_who(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		status[2]=0;
 		sendto_one(sptr, getreply(RPL_WHOREPLY), me.name, sptr->name,
 			   wsopts.show_chan ? first_visible_channel(ac, sptr)
-			   : "*", ac->user->username, WHO_HOST(ac),
+			   : "*", ac->user->username, WHO_HOST(sptr,ac),
 			   WHO_SERVER(sptr, ac), ac->name, status,
 			   WHO_HOPCOUNT(sptr, ac),
 			   ac->info);
@@ -811,7 +858,7 @@ int m_who(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	    for(cm = lp->value.chptr->members; cm; cm = cm->next)
 	    {
 		ac = cm->cptr;
-		if(!chk_who(ac, 1))
+		if(!chk_who(ac, sptr, 1))
 		    continue;
 		
 		if(shown==MAXWHOREPLIES && !IsAnOper(sptr))
@@ -831,7 +878,7 @@ int m_who(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		status[++i]=0;
 		sendto_one(sptr, getreply(RPL_WHOREPLY), me.name, sptr->name,
 			   lp->value.chptr->chname, ac->user->username,
-			   WHO_HOST(ac),WHO_SERVER(sptr, ac), ac->name,
+			   WHO_HOST(sptr,ac),WHO_SERVER(sptr, ac), ac->name,
 			   status, WHO_HOPCOUNT(sptr, ac), ac->info);
 		shown++;
 	    }
@@ -841,7 +888,7 @@ int m_who(aClient *cptr, aClient *sptr, int parc, char *parv[])
     {
 	for(ac=client;ac;ac=ac->next)
 	{
-	    if(!chk_who(ac,showall))
+	    if(!chk_who(ac,sptr,showall))
 		continue;
 	    /* wow, they passed it all, give them the reply...
 	     * IF they haven't reached the max, or they're an oper */
@@ -857,7 +904,7 @@ int m_who(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	    status[2]=0;
 	    sendto_one(sptr, getreply(RPL_WHOREPLY), me.name, sptr->name,
 		       wsopts.show_chan ? first_visible_channel(ac, sptr) :
-		       "*", ac->user->username, WHO_HOST(ac),
+		       "*", ac->user->username, WHO_HOST(sptr,ac),
 		       WHO_SERVER(sptr, ac), ac->name, status,
 		       WHO_HOPCOUNT(sptr, ac), ac->info);
 	    shown++;
