@@ -478,6 +478,7 @@ typedef struct module_hook
 } aHook;
 
 static DLink *preaccess_hooks = NULL;
+static DLink *onaccess_hooks = NULL;
 static DLink *postaccess_hooks = NULL;
 static DLink *postmotd_hooks = NULL;
 static DLink *msg_hooks = NULL;
@@ -489,6 +490,10 @@ static DLink *join_hooks = NULL;
 static DLink *sendburst_hooks = NULL;
 static DLink *throttle_hooks = NULL;
 static DLink *forbid_hooks = NULL;
+static DLink *whois_hooks = NULL;
+static DLink *maskhost_hooks = NULL;
+static DLink *floodwarn_hooks = NULL;
+static DLink *spamwarn_hooks = NULL;
 static DLink *signoff_hooks = NULL;
 static DLink *mload_hooks = NULL;
 static DLink *munload_hooks = NULL;
@@ -508,6 +513,9 @@ get_texthooktype(enum c_hooktype hooktype)
         case CHOOK_PREACCESS:
             return "Pre-access";
 
+        case CHOOK_ONACCESS:
+            return "On-access";
+
         case CHOOK_POSTACCESS:
             return "Post-access";
 
@@ -538,6 +546,18 @@ get_texthooktype(enum c_hooktype hooktype)
         case CHOOK_FORBID:
             return "forbid";
 
+        case CHOOK_WHOIS:
+            return "whois";
+
+        case CHOOK_MASKHOST:
+            return "Mask Host";
+
+        case CHOOK_FLOODWARN:
+            return "FloodWarn";
+
+        case CHOOK_SPAMWARN:
+            return "SpamWarn";
+
         case CHOOK_SIGNOFF:
             return "Signoff";
 
@@ -568,6 +588,10 @@ get_hooklist(enum c_hooktype hooktype)
             hooklist = &preaccess_hooks;
             break;
 
+        case CHOOK_ONACCESS:
+            hooklist = &onaccess_hooks;
+            break;
+
         case CHOOK_POSTACCESS:
             hooklist = &postaccess_hooks;
             break;
@@ -608,6 +632,22 @@ get_hooklist(enum c_hooktype hooktype)
             hooklist = &forbid_hooks;
             break;
 
+        case CHOOK_WHOIS:
+            hooklist = &whois_hooks;
+            break;
+
+        case CHOOK_MASKHOST:
+            hooklist = &maskhost_hooks;
+            break;
+
+        case CHOOK_FLOODWARN:
+            hooklist = &floodwarn_hooks;
+            break;
+
+        case CHOOK_SPAMWARN:
+            hooklist = &spamwarn_hooks;
+            break;
+
         case CHOOK_SIGNOFF:
             hooklist = &signoff_hooks;
             break;
@@ -734,6 +774,24 @@ call_hooks(enum c_hooktype hooktype, ...)
                 break;
             }
 
+        case CHOOK_ONACCESS:
+            {
+                aClient *acptr = va_arg(vl, aClient *);
+                char *username = va_arg(vl, char *);
+                char *host = va_arg(vl, char *);
+                char *server = va_arg(vl, char *);
+                char *realname = va_arg(vl, char *);
+
+                for(lp = onaccess_hooks; lp; lp = lp->next)
+                {
+                    int (*rfunc) (aClient *, char *, char *, char *, char *) =
+                                    ((aHook *)lp->value.cp)->funcptr;
+                    if((ret = (*rfunc)(acptr, username, host, server, realname)) == FLUSH_BUFFER)
+                        break;
+                }
+                break;
+            }
+
         case CHOOK_POSTACCESS:
             {
                 aClient *acptr = va_arg(vl, aClient *);
@@ -887,6 +945,75 @@ call_hooks(enum c_hooktype hooktype, ...)
                 break;
             }
 
+        case CHOOK_WHOIS:
+            {
+                aClient *sptr = va_arg(vl, aClient *);
+                aClient *acptr = va_arg(vl, aClient *);
+                for(lp = whois_hooks; lp; lp = lp->next)
+                {
+                    int (*rfunc) (aClient *, aClient *) = 
+                                    ((aHook *)lp->value.cp)->funcptr;
+                    if((ret = (*rfunc)(sptr, acptr)) == FLUSH_BUFFER)
+                        break;
+                }
+                break;
+            }
+
+
+        case CHOOK_MASKHOST:
+            {
+                char *orghost = va_arg(vl, char *);
+                char *orgip = va_arg(vl, char *);
+                char *newhost = va_arg(vl, char *);
+                int type = va_arg(vl, int);
+                for(lp = maskhost_hooks; lp; lp = lp->next)
+                {
+                    int (*rfunc) (char *, char *, char **, int) = 
+                                    ((aHook *)lp->value.cp)->funcptr;
+                    /* Possible results by the module:
+                       1 (UHM_SUCCESS)                      = Success, the host has been masked (so don't try other modules).
+                       0 (UHM_SOFT_FAILURE)                 = Failure, the host wasn't masked but try other modules (maybe they will mask the host).
+                       -2 (UHM_HARD_FAILURE / FLUSH_BUFFER) = Failure, the host wasn't masked but *don't* try other modules.
+                     */
+                    if((ret = (*rfunc)(orghost, orgip, &newhost, type)) != UHM_SOFT_FAILURE)
+                        break; /* We stop trying other modules if we get UHM_SUCCESS or UHM_HARD_FAILURE */
+                }
+                break;
+            }
+
+        case CHOOK_FLOODWARN:
+            {
+                aClient *sptr = va_arg(vl, aClient *);
+                aChannel *chptr = va_arg(vl, aChannel *);
+                int type = va_arg(vl, int);
+                char *cmd = va_arg(vl, char *);
+                char *reason = va_arg(vl, char *);
+                for(lp = floodwarn_hooks; lp; lp = lp->next)
+                {
+                    int (*rfunc) (aClient *, aChannel *, int, char *, char *) = 
+                                    ((aHook *)lp->value.cp)->funcptr;
+                    if((ret = (*rfunc)(sptr, chptr, type, cmd, reason)) == FLUSH_BUFFER)
+                        break;
+                }
+                break;
+            }
+
+        case CHOOK_SPAMWARN:
+            {
+                aClient *sptr = va_arg(vl, aClient *);
+                int type = va_arg(vl, int);
+                int max_targets = va_arg(vl, int);
+                char *target_name = va_arg(vl, char *);
+                for(lp = spamwarn_hooks; lp; lp = lp->next)
+                {
+                    int (*rfunc) (aClient *, int, int, char *) = 
+                                    ((aHook *)lp->value.cp)->funcptr;
+                    if((ret = (*rfunc)(sptr, type, max_targets, target_name)) == FLUSH_BUFFER)
+                        break;
+                }
+                break;
+            }
+
         case CHOOK_SIGNOFF:
             {
                 aClient *acptr = va_arg(vl, aClient *);
@@ -999,6 +1126,7 @@ memcount_modules(MCmodules *mc)
     mc->e_dlinks += c;
 
     mc->e_dlinks += mc_dlinks(preaccess_hooks);
+    mc->e_dlinks += mc_dlinks(onaccess_hooks);
     mc->e_dlinks += mc_dlinks(postaccess_hooks);
     mc->e_dlinks += mc_dlinks(postmotd_hooks);
     mc->e_dlinks += mc_dlinks(msg_hooks);
@@ -1010,6 +1138,10 @@ memcount_modules(MCmodules *mc)
     mc->e_dlinks += mc_dlinks(sendburst_hooks);
     mc->e_dlinks += mc_dlinks(throttle_hooks);
     mc->e_dlinks += mc_dlinks(forbid_hooks);
+    mc->e_dlinks += mc_dlinks(whois_hooks);
+    mc->e_dlinks += mc_dlinks(maskhost_hooks);
+    mc->e_dlinks += mc_dlinks(floodwarn_hooks);
+    mc->e_dlinks += mc_dlinks(spamwarn_hooks);
     mc->e_dlinks += mc_dlinks(signoff_hooks);
     mc->e_dlinks += mc_dlinks(mload_hooks);
     mc->e_dlinks += mc_dlinks(munload_hooks);
