@@ -26,8 +26,9 @@ static char *replies[] =
     /* 002 RPL_YOURHOST */	":%s 002 %s :Your host is %s, running "
                                 "version %s",
     /* 003 RPL_CREATED */	":%s 003 %s :This server was created %s",
-    /* 004 RPL_MYINFO */	":%s 004 %s %s %s aAbcCdefFghiIjkKmnoOrRsSwxXy "
-                                "AbceiIjklLmMnoOprRsStv",
+    /* 004 RPL_MYINFO */	":%s 004 %s %s %s aAbcCdefFghHiIjkKmnoOPrRsSwxXy "
+                                "AbceiIjklLmMnoOpPrRsStv "
+                                "beIjklov",
     /* 005 RPL_ISUPPORT */      ":%s 005 %s %s :are available on this server",
     /* 006 */	                NULL,
     /* 007 */	                NULL,
@@ -237,8 +238,8 @@ static char *replies[] =
     /* 211 */	                NULL,
     /* 212 RPL_STATSCOMMANDS */	":%s 212 %s %s %u %u",
     /* 213 RPL_STATSCLINE */	":%s 213 %s %s %s * %s %d %s",
-    /* 214 RPL_STATSNLINE */	":%s 214 %s %s %s * %s %d/%d %s",
-    /* 215 RPL_STATSILINE */	":%s 215 %s %s %s %d %s %d %s",
+    /* 214 RPL_STATSNLINE */	":%s 214 %s %s %s * %s %s %s",
+    /* 215 RPL_STATSILINE */	":%s 215 %s %s %s %s %s %d %s",
     /* 216 RPL_STATSKLINE */	":%s 216 %s %s %s * %s %d :%s",
     /* 217 RPL_STATSQLINE */	":%s 217 %s %s * * %s %d :%s",
     /* 218 RPL_STATSYLINE */	":%s 218 %s %c %s %d %d:%d %d %ld",
@@ -249,7 +250,7 @@ static char *replies[] =
     /* 223 */			":%s 223 %s %c %s * %s %d %d",
     /* 224 */			":%s 224 %s %c %s * %s %d %d",
     /* 225 RPL_STATSCLONE */	":%s 225 %s D %s %d %d %d",
-    /* 226 RPL_STATSCOUNT */	":%s 226 %s %s %l",
+    /* 226 RPL_STATSCOUNT */	":%s 226 %s %s %lu",
     /* 227 RPL_STATSGLINE */	":%s 227 %s %s * * %s %d :%s",
     /* 228 */	                NULL,
     /* 229 */	                NULL,
@@ -266,9 +267,9 @@ static char *replies[] =
     /* 240 */	                NULL,
     /* 241 RPL_STATSLLINE */	":%s 241 %s %c %s * %s %d %d",
     /* 242 RPL_STATSUPTIME */	":%s 242 %s :Server Up %d days, %d:%02d:%02d",
-    /* 243 RPL_STATSOLINE */	":%s 243 %s %s %s * %s %lu %s",
+    /* 243 RPL_STATSOLINE */	":%s 243 %s %s %s * %s %s %s",
     /* 244 RPL_STATSHLINE */	NULL,
-    /* 245 RPL_STATSSLINE */	NULL,
+    /* 245 RPL_STATSSLINE */	":%s 245 %s %s %s %ld %s %lu :%s",
     /* 246 RPL_STATSXLINE */	":%s 246 %s %s %s * %s %d %d",
     /* 247 */	                NULL,		/* Undernet's STATSGLINE */
     /* 248 */	                NULL,		/* Undernet's STATSULINE */
@@ -418,7 +419,7 @@ static char *replies[] =
     /* 376 RPL_ENDOFMOTD */	":%s 376 %s :End of /MOTD command.",
     /* 377 */	                NULL,
     /* 378 */	                NULL,
-    /* 379 */	                NULL,
+    /* 379 RPL_WHOISMODES */	":%s 379 %s %s :has the modes: %s",
     /* 380 */	                NULL,
     /* 381 RPL_YOUREOPER */	":%s 381 %s :You are now an IRC Operator",
     /* 382 RPL_REHASHING */	":%s 382 %s %s :Rehashing",
@@ -534,14 +535,14 @@ static char *replies[] =
     /* 475 ERR_BADCHANNELKEY */	":%s 475 %s %s :Cannot join channel (+k)",
     /* 476 ERR_BADCHANMASK */	":%s 476 %s %s :Bad Channel Mask",
     /* 477 ERR_NEEDREGGEDNICK */       ":%s 477 %s %s :You need to identify "
-                                       "to a registered nick to %s that "
-                                       "channel. For help with registering "
+                                       "to a registered nick to %s "
+                                       "%s. For help with registering "
                                        "your nickname, type \"/msg %s@%s help "
                                        "register\" or see %s",
     /* 478 ERR_BANLISTFULL */	":%s 478 %s %s %s :Channel %s list is full",
     /* 479 ERR_BADCHANNAME */   ":%s 479 %s %s :Channel name contains illegal "
                                 "characters",
-    /* 480 */	                NULL,
+    /* 480 ERR_NEEDTOWAIT */	":%s 480 %s :You need to wait %ld seconds before being able to talk in %s",
     /* 481 ERR_NOPRIVILEGES */	":%s 481 %s :Permission Denied, You do not "
                                 "have the correct irc operator privileges",
     /* 482 ERR_CHANOPRIVSNEEDED */	":%s 482 %s %s :You're not channel "
@@ -556,7 +557,7 @@ static char *replies[] =
     /* 488 ERR_NOSSL */	        ":%s 488 %s :SSL Only channel (+S), You must connect "
                                 "using SSL to join this channel.",
     /* 489 */	                NULL,	/* In use by Undernet */
-    /* 490 */	                NULL,
+    /* 490 ERR_MAXMSGSENT */    ":%s 490 %s :You must wait %ld seconds before you can send another message to %s",
     /* 491 ERR_NOOPERHOST */	":%s 491 %s :No Oper block for your host",
     /* 492 */	                NULL,
     /* 493 ERR_NOSHAREDCHAN */	":%s 493 %s :You cannot message that person because you do not share a common channel with them.",
